package UI.LogIn.NetWorkGame;

import UI.GameState.EnemyTank;
import UI.GameState.MyTank;
import UI.GameState.WallState;
import UI.LogIn.Game.Rendering;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferStrategy;
import java.io.IOException;
import java.util.ArrayList;

public class NWGFrame extends JFrame {

    private long lastRender;
    private BufferStrategy bufferStrategy;
    private NWGRendering rendering = new NWGRendering();
    public static final int GAME_HEIGHT = 720;                  // 720p game resolution
    public static final int GAME_WIDTH = 16 * GAME_HEIGHT / 9;  // wide aspect ratio
    public WallState wallState = new WallState(GAME_WIDTH, GAME_HEIGHT);


    public NWGFrame(String gameName) {
        lastRender = -1;
        setTitle(gameName);
        pack();
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setSize(GAME_WIDTH, GAME_HEIGHT);
        setResizable(false);

    }

    /**
     * This must be called once after the JFrame is shown:
     * frame.setVisible(true);
     * and before any rendering is started.
     */
    public void initBufferStrategy() {
        // Triple-buffering
        createBufferStrategy(3);
        bufferStrategy = getBufferStrategy();
    }

    /**
     * Game rendering with triple-buffering using BufferStrategy.
     */

    public void render(MyTank myTank, ArrayList<EnemyTank> enemyTanks , WallState wallState) {
        do {
            do {
                Graphics2D graphics = (Graphics2D) bufferStrategy.getDrawGraphics();

                try {
                    rendering.doRendering(graphics, myTank, enemyTanks , wallState );
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    // Dispose the graphics
                    graphics.dispose();
                }
                // Repeat the rendering if the drawing buffer contents were restored
            } while (bufferStrategy.contentsRestored());

            // Display the buffer
            bufferStrategy.show();
            // Tell the system to do the drawing NOW;
            // otherwise it can take a few extra ms and will feel jerky!
            Toolkit.getDefaultToolkit().sync();

            // Repeat the rendering if the drawing buffer was lost
        } while (bufferStrategy.contentsLost());
    }
}
